@@ -18,13 +18,10 @@ from collections.abc import Mapping, Sequence, Iterable, Callable
 from debian.debian_support import DpkgArchTable
 
 from debputy._deb_options_profiles import DebBuildOptionsAndProfiles
-from debputy.architecture_support import (
-    DpkgArchitectureBuildProcessValuesTable,
-    dpkg_architecture_table,
-)
+from debputy.architecture_support import DpkgArchitectureBuildProcessValuesTable
 from debputy.dh.dh_assistant import read_dh_addon_sequences
 from debputy.exceptions import DebputyRuntimeError
-from debputy.filesystem_scan import FSROOverlay
+from debputy.filesystem_scan import OSFSROOverlay
 from debputy.highlevel_manifest import HighLevelManifest
 from debputy.highlevel_manifest_parser import YAMLManifestParser
 from debputy.integration_detection import determine_debputy_integration_mode
@@ -88,7 +85,7 @@ class Command:
     requested_plugins_only: bool = False
 
 
-def _host_dpo_to_dbo(opt_and_profiles: "DebBuildOptionsAndProfiles", v: str) -> bool:
+def _host_dpo_to_dbo(opt_and_profiles: DebBuildOptionsAndProfiles, v: str) -> None:
 
     if (
         v in opt_and_profiles.deb_build_profiles
@@ -99,9 +96,6 @@ def _host_dpo_to_dbo(opt_and_profiles: "DebBuildOptionsAndProfiles", v: str) ->
             f'Copying "{v}" into DEB_BUILD_OPTIONS: It was in DEB_BUILD_PROFILES but not in DEB_BUILD_OPTIONS'
         )
         os.environ["DEB_BUILD_OPTIONS"] = val.lstrip()
-        # Note: It will not be immediately visible since `DebBuildOptionsAndProfiles` caches the result
-        return True
-    return False
 
 
 class CommandContext:
@@ -119,7 +113,7 @@ class CommandContext:
         self._debputy_plugin_feature_set: PluginProvidedFeatureSet = (
             PluginProvidedFeatureSet()
         )
-        self._debian_dir = FSROOverlay.create_root_dir("debian", "debian")
+        self._debian_dir = OSFSROOverlay.create_root_dir("debian", "debian")
         self._mtime: int | None = None
         self._source_variables: Mapping[str, str] | None = None
         self._substitution: Substitution | None = None
@@ -168,13 +162,10 @@ class CommandContext:
             if hasattr(self.parsed_args, "packages"):
                 packages = self.parsed_args.packages
 
-            instance = DebBuildOptionsAndProfiles(environ=os.environ)
+            instance = DebBuildOptionsAndProfiles()
 
-            dirty = _host_dpo_to_dbo(instance, "nodoc")
-            dirty = _host_dpo_to_dbo(instance, "nocheck") or dirty
-
-            if dirty:
-                instance = DebBuildOptionsAndProfiles(environ=os.environ)
+            _host_dpo_to_dbo(instance, "nodoc")
+            _host_dpo_to_dbo(instance, "nocheck")
 
             parser = DctrlParser(
                 packages,  # -p/--package
@@ -183,7 +174,7 @@ class CommandContext:
                 self._package_set == "indep",
                 self._package_set == "arch",
                 deb_options_and_profiles=instance,
-                dpkg_architecture_variables=dpkg_architecture_table(),
+                dpkg_architecture_variables=DpkgArchitectureBuildProcessValuesTable(),
                 dpkg_arch_query_table=DpkgArchTable.load_arch_table(),
             )
             self._dctrl_parser = parser
@@ -209,7 +200,7 @@ class CommandContext:
         return set(getattr(self.parsed_args, "required_plugins") or [])
 
     @property
-    def deb_build_options_and_profiles(self) -> "DebBuildOptionsAndProfiles":
+    def deb_build_options_and_profiles(self) -> DebBuildOptionsAndProfiles:
         return self.dctrl_parser.deb_options_and_profiles
 
     @property
