@@ -467,7 +467,7 @@ def classify_debian_packaging_files(
 
     paths = []
 
-    for entry in debian_dir.iterdir:
+    for entry in debian_dir.iterdir():
         if entry.is_dir or entry.name.startswith("."):
             continue
         if entry.path in ignore_paths:
