@@ -1,43 +1,41 @@
 import contextlib
 import dataclasses
-import os
+import io
 import tempfile
 import textwrap
 from typing import (
     Optional,
     Literal,
-    Union,
     overload,
-    FrozenSet,
     TypeVar,
     Any,
     TYPE_CHECKING,
     TextIO,
-    BinaryIO,
     Generic,
     ContextManager,
-    List,
-    Type,
-    Tuple,
     get_args,
     final,
     cast,
 )
 from collections.abc import Iterable, Callable, Iterator, Sequence, Container
 
+from debian.debian_support import DpkgArchTable
 from debian.substvars import Substvars
 
 from debputy import util
+from debputy._deb_options_profiles import DebBuildOptionsAndProfiles
 from debputy.exceptions import (
     TestPathWithNonExistentFSPathError,
     PureVirtualPathError,
     PluginInitializationError,
 )
 from debputy.interpreter import Interpreter, extract_shebang_interpreter_from_file
+from debputy.manifest_conditions import ConditionContext
 from debputy.manifest_parser.tagging_types import DebputyDispatchableType
 from debputy.manifest_parser.util import parse_symbolic_mode
-from debputy.packages import BinaryPackage
+from debputy.packages import BinaryPackage, SourcePackage
 from debputy.types import S
+from debputy.util import PackageTypeSelector
 
 if TYPE_CHECKING:
     from debputy.manifest_parser.base_types import (
@@ -49,10 +47,6 @@ if TYPE_CHECKING:
     from debputy.plugins.debputy.to_be_api_types import BuildSystemRule
     from debputy.plugin.api.impl import DebputyPluginInitializerProvider
 
-    # Work around it being "unused" since `cast("...", ...)` does not register as a use
-    # for most static analysis tools.
-    assert DebputyPluginInitializerProvider
-
 
 DP = TypeVar("DP", bound=DebputyDispatchableType)
 
@@ -71,7 +65,7 @@ DpkgTriggerType = Literal[
     "interest-noawait",
 ]
 Maintscript = Literal["postinst", "preinst", "prerm", "postrm"]
-PackageTypeSelector = Union[Literal["deb", "udeb"], Iterable[Literal["deb", "udeb"]]]
+
 ServiceUpgradeRule = Literal[
     "do-nothing",
     "reload",
@@ -332,6 +326,11 @@ class PackageProcessingContext:
 
     __slots__ = ()
 
+    @property
+    def source_package(self) -> SourcePackage:
+        """The source package stanza from `debian/control`"""
+        raise NotImplementedError
+
     @property
     def binary_package(self) -> BinaryPackage:
         """The binary package stanza from `debian/control`"""
@@ -361,8 +360,44 @@ class PackageProcessingContext:
     def accessible_package_roots(self) -> Iterable[tuple[BinaryPackage, "VirtualPath"]]:
         raise NotImplementedError
 
-    # """The source package stanza from `debian/control`"""
-    # source_package: SourcePackage
+    def manifest_configuration[T](
+        self,
+        context_package: SourcePackage | BinaryPackage,
+        value_type: type[T],
+    ) -> T | None:
+        """Request access to configuration from the manifest
+
+        This method will return the value associated with a pluggable manifest rule assuming
+        said configuration was provided.
+
+
+        :param context_package: The context in which the configuration will be. Generally, it will be
+          the binary package for anything under `packages:` and the source package otherwise.
+        :param value_type: The type used during registered (return type of the parser/unpack function)
+        :return: The value from the manifest if present or `None` otherwise
+        """
+        raise NotImplementedError
+
+    @property
+    def dpkg_arch_query_table(self) -> DpkgArchTable:
+        raise NotImplementedError
+
+    @property
+    def deb_options_and_profiles(self) -> DebBuildOptionsAndProfiles:
+        raise NotImplementedError
+
+    @property
+    def source_condition_context(self) -> ConditionContext:
+        raise NotImplementedError
+
+    def condition_context(
+        self, binary_package: BinaryPackage | None
+    ) -> ConditionContext:
+        raise NotImplementedError
+
+    @property
+    def binary_condition_context(self) -> ConditionContext:
+        return self.condition_context(self.binary_package)
 
 
 class DebputyPluginDefinition:
@@ -392,7 +427,7 @@ class DebputyPluginDefinition:
         func: MetadataAutoDetector | None = None,
         *,
         detector_id: str | None = None,
-        package_type: PackageTypeSelector = "deb",
+        package_types: PackageTypeSelector = PackageTypeSelector.DEB,
     ) -> Callable[[MetadataAutoDetector], MetadataAutoDetector] | MetadataAutoDetector:
         """Provide a pre-assembly hook that can affect the metadata/maintscript of binary ("deb") packages
 
@@ -433,7 +468,7 @@ class DebputyPluginDefinition:
         :param func: The function to be decorated/registered.
         :param detector_id: A plugin-wide unique ID for this detector. Packagers may use this ID for disabling
           the detector and accordingly the ID is part of the plugin's API toward the packager.
-        :param package_type: Which kind of packages this metadata detector applies to.  The package type is generally
+        :param package_types: Which kind of packages this metadata detector applies to.  The package type is generally
           defined by `Package-Type` field in the binary package. The default is to only run for regular `deb` packages
           and ignore `udeb` packages.
         """
@@ -446,7 +481,7 @@ class DebputyPluginDefinition:
                 api.metadata_or_maintscript_detector(
                     final_id,
                     f,
-                    package_type=package_type,
+                    package_types=package_types,
                 )
 
             self._generic_initializers.append(_init)
@@ -718,7 +753,7 @@ class DebputyPluginInitializer:
         auto_detector_id: str,
         auto_detector: MetadataAutoDetector,
         *,
-        package_type: PackageTypeSelector = "deb",
+        package_types: PackageTypeSelector = PackageTypeSelector.DEB,
     ) -> None:
         """Provide a pre-assembly hook that can affect the metadata/maintscript of binary ("deb") packages
 
@@ -741,7 +776,7 @@ class DebputyPluginInitializer:
           the detector and accordingly the ID is part of the plugin's API toward the packager.
         :param auto_detector: The code to be called that will be run at the metadata generation state (once for each
           binary package).
-        :param package_type: Which kind of packages this metadata detector applies to.  The package type is generally
+        :param package_types: Which kind of packages this metadata detector applies to.  The package type is generally
           defined by `Package-Type` field in the binary package. The default is to only run for regular `deb` packages
           and ignore `udeb` packages.
         """
@@ -1055,7 +1090,6 @@ class VirtualPath:
         """
         raise NotImplementedError
 
-    @property
     def iterdir(self) -> Iterable["VirtualPath"]:
         """Returns an iterable that iterates over all children of this path
 
@@ -1106,23 +1140,6 @@ class VirtualPath:
         """
         raise NotImplementedError
 
-    @property
-    def is_detached(self) -> bool:
-        """Returns True if this path is detached
-
-        Paths that are detached from the file system will not be present in the package and
-        most operations are unsafe on them. This usually only happens if the path or one of
-        its parent directories are unlinked (rm'ed) from the file system tree.
-
-        All paths are attached by default and will only become detached as a result of
-        an action to mutate the virtual file system.  Note that the file system may not
-        always be manipulated.
-
-        :return: True if the entry is detached. Detached entries should be discarded, so they
-        can be garbage collected.
-        """
-        raise NotImplementedError
-
     # The __getitem__ behaves like __getitem__ from Dict but __iter__ would ideally work like a Sequence.
     # However, that does not feel compatible, so lets force people to use .children instead for the Sequence
     # behavior to avoid surprises for now.
@@ -1208,6 +1225,14 @@ class VirtualPath:
             return f"/{p}"
         return p
 
+    def is_root_dir(self) -> bool:
+        """Whether the current path is the root directory
+
+        :return: True if this path is the root directory. False otherwise
+        """
+        # The root directory is never detachable in the current setup
+        raise NotImplementedError
+
     @property
     def parent_dir(self) -> Optional["VirtualPath"]:
         """The parent directory of this path
@@ -1219,30 +1244,13 @@ class VirtualPath:
         """
         raise NotImplementedError
 
-    def stat(self) -> os.stat_result:
-        """Attempt to do stat of the underlying path (if it exists)
-
-        *Avoid* using `stat()` whenever possible where a more specialized attribute exist.  The
-        `stat()` call returns the data from the file system and often, `debputy` does *not* track
-        its state in the file system.  As an example, if you want to know the file system mode of
-        a path, please use the `mode` attribute instead.
-
-        This never follow symlinks (it behaves like `os.lstat`). It will raise an error
-        if the path is not backed by a file system object (that is, `has_fs_path` is False).
-
-        :return: The stat result or an error.
-        """
-        raise NotImplementedError()
-
     @property
     def size(self) -> int:
         """Resolve the file size (`st_size`)
 
-        This may be using `stat()` and therefore `fs_path`.
-
         :return: The size of the file in bytes
         """
-        return self.stat().st_size
+        raise NotImplementedError
 
     @property
     def mode(self) -> int:
@@ -1378,23 +1386,18 @@ class VirtualPath:
         self,
         *,
         byte_io: Literal[True],
-        buffering: int = -1,
-    ) -> BinaryIO: ...
+        buffering: Literal[0] = ...,
+    ) -> io.FileIO: ...
 
     @overload
     def open(
         self,
         *,
-        byte_io: bool,
+        byte_io: Literal[True],
         buffering: int = -1,
-    ) -> TextIO | BinaryIO: ...
+    ) -> io.BufferedReader: ...
 
-    def open(
-        self,
-        *,
-        byte_io: bool = False,
-        buffering: int = -1,
-    ) -> TextIO | BinaryIO:
+    def open(self, *, byte_io=False, buffering=-1):
         """Open the file for reading.  Usually used with a context manager
 
         By default, the file is opened in text mode (utf-8). Binary mode can be requested
@@ -1920,7 +1923,7 @@ def documented_attr(
 
 
 def reference_documentation(
-    title: str = "Auto-generated reference documentation for ${RULE_NAME}",
+    title: str | None = None,
     description: str | None = textwrap.dedent(
         """\
             This is an automatically generated reference documentation for ${RULE_NAME}. It is generated
@@ -1964,6 +1967,8 @@ def reference_documentation(
     :param reference_documentation_url: A URL to the reference documentation.
     :return: An opaque representation of the documentation,
     """
+    if title is None:
+        title = "Auto-generated reference documentation for ${RULE_NAME}"
     return ParserDocumentation(
         synopsis,
         title,
